<?php
//////////////////////////////////////////////////////////////////////////
//
// Class to query the eBay API
//  Base class for all ebay api calls.
//
//////////////////////////////////////////////////////////////////////////

require_once __DIR__."/constants.php";
require_once __DIR__."/utilities.php";

if (file_exists(__DIR__."/tracking_pro.php"))
  include_once __DIR__."/tracking_pro.php";
else
  include_once __DIR__."/tracking.php";
  
abstract class fuEbayApiCall
{
  use fuEbayTracking; 

  private const FEL_ENDPOINT = "https://felp.fubaby.com/";

  protected static $id = 0;
  protected $globalId;
  protected $deferLoading;
  protected $disableApiCache;
  protected $requestUri;
  protected $result = "";
  protected $resultCount = 0;
  
  public $enableTraceLog = false;
  public $show = fuEbayShow::All;

  // Core properties to adjust presentation.
  public $customId = '';
  public $picWidth = 0;
  public $arrangement = 0;
  
  function __construct() 
  {
    $this->globalId = get_option('fubaby_ebayGlobalID');
    $this->deferLoading = get_option('fubaby_ebayDeferredLoading', "NotCached");
    $this->disableApiCache = get_option('fubaby_ebayDisableAPICache', false);
    $this->enableTraceLog = get_option('fubaby_ebayTraceLogging', false);
  }

  abstract protected function initEbayListing($item);
  
  // Utility funcs.
  protected function traceLog(string $msg)
  {
    if ($this->enableTraceLog)
      $this->result .= "<!-- " . FUBABY_EBAY_PLUGIN_TITLE . " Trace Log: {$msg} -->\r\n";
  }

  protected function errorLog(string $msg)
  {
    error_log(FUBABY_EBAY_PLUGIN_TITLE . ": {$msg}");
  }  
  
  protected function getEndpoint() : string
  {
    $endpointOverride = get_option('fubaby_ebayFelServerOverride');
    return empty($endpointOverride) ? fuEbayApiCall::FEL_ENDPOINT : $endpointOverride;
  }

  protected function getRequestUri() : string
  {
    return $this->requestUri;
  }
  
  public function getResultCount() : int
  {
    return $this->resultCount;
  }
  
  public function forceDeferredLoading()
  {
    $this->deferLoading = "Always";
  }

  public function disableDeferredLoading()
  {
    $this->deferLoading = "Never";
  }

  protected function validateArgs()
  {  
    $this->picWidth = intval($this->picWidth);
  }

  private function getCallHash()
  {
    return "fuEbay_" . md5($this->requestUri);
  }

  protected function constructRequest()
  { 
    $this->requestUri .= "&ver=" . FUBABY_EBAY_PLUGIN_VER;
    
    $siteHostname = parse_url(get_site_url(), PHP_URL_HOST);
    $this->requestUri .= "&hostname={$siteHostname}";

    $subscriptionKey = get_option('fubaby_ebaySubscriptionKey');
    if (!empty($subscriptionKey))
      $this->requestUri .= "&subkey={$subscriptionKey}";

    $globalId = str_replace("-", "_", get_option('fubaby_ebayGlobalID'));	
    $this->requestUri .= "&globalId={$globalId}";

    $campaignId = get_option('fubaby_ebayCampaignID');
    if (!empty($campaignId) && preg_match_all("/[0-9]/", $campaignId) == 10)
      $this->requestUri .= "&affiliateCampaignId={$campaignId}";

    $referenceId = !empty($this->customId) ? $this->customId : get_option('fubaby_ebayDefCustomID');
    if (!empty($referenceId))
      $this->requestUri .= "&affiliateReferenceId=".urlencode($referenceId)."";
  }

   
  public function call($fromAjax = false)
  {
    $this->result = "";    
    if (!$this->validateArgs())
      return FUBABY_EBAY_PLUGIN_TITLE . ": Bad arguments";

    if (empty($this->requestUri))
    {
      $this->constructRequest();
      $this->traceLog("Request URI: " . $this->requestUri);
    }

    // Fetch the request, either now or defer until after page load
    if ($fromAjax ||
      $this->deferLoading == "Never" ||
      ($this->deferLoading == "NotCached" && $this->isRequestCached()))
    {
      $resp = $this->fetch();
      $this->createListingGrid($resp);
    }
    else
    {
      $this->createDeferredLoadingStub();
    }
  
    return $this->result;  
  }

  protected function isRequestCached()
  {
    $callHash = $this->getCallHash();
    $resp = get_transient($callHash);
    $this->traceLog("Request cached: " . ($resp !== false ? "True" : "False"));
    return $resp !== false;
  }

  protected function fetch()
  {
    // Retrieve cached transient
    $callHash = $this->getCallHash();
    $resp = get_transient($callHash);
    if ($resp !== false && !$this->disableApiCache) 
    {
      $this->traceLog(__("Retrieved pre-cached transient."));
      $result = json_decode($resp);
    }
    else
    {
      // No cached transient, so go query remotely.
      $timeStart = microtime(true);
      $resp = wp_remote_retrieve_body(wp_remote_get($this->requestUri));
      if ($resp == null)
      {
        $this->errorLog(__("Failed to query eBay API."));
        return null;
      }

      $result = json_decode($resp);
      if (isset($result->statusCode) && $result->statusCode != 200)
      {
        $this->errorLog(__("Failed to query eBay API."));
        return null;
      }

      // Only save transient on successful fetch
      set_transient($callHash, $resp, FUBABY_EBAY_CALLCACHEEXPIRY * MINUTE_IN_SECONDS);
      $timeEnd = microtime(true);
      $this->traceLog("eBay API call took: " . ($timeEnd - $timeStart) . " secs.");
    }

    return $result;
  }
}