<?php

//////////////////////////////////////////////////////////////////////////
//
// Class to query the eBay Browse API Search function
//
//////////////////////////////////////////////////////////////////////////

require_once __DIR__."/apicall.php";
require_once __DIR__."/presentation.php";

class fuEbayBrowseSearchApiCall extends fuEbayApiCall
{
  const EBAY_LIMIT_MAX = 200;
  const EBAY_CATEGORIES_MAX = 1;

  use fuEbayTable;
  
  protected $title = "";
  protected $categoryIds = array();
  protected $aspectName = "";
  protected $aspectValue = "";
  protected $query = "";
  protected $sellers = array();
  protected $sortOrder = "";
  
  public $minPrice = -1;
  public $maxPrice = -1;
       
  function __construct($title, $requestUri = "") 
  {
    parent::__construct();
    $this->arrangement = get_option('fubaby_ebayDefArrangementList', 0);
    $this->title = $title;
    $this->requestUri = $requestUri;
  }
  
  public function setQuery(string $query)
  {
    $this->query = $query;
  }
  
  public function setSellers(string $seller)
  {
    switch (strtolower($seller))
    {
      case "all":
        $this->sellers = array();
        break;
      case "author":
        $seller = get_the_author_meta('ebayuserid');
        // fall through
      default:  
        if (empty($seller))
          $seller = get_option('fubaby_ebayDefSeller');
        $this->sellers = !empty($seller) ? explode(',', $seller) : array();
    }
  }
  
  public function setSortOrder(string $sortOrder)
  {
    global $ebay_map_sort_browseapi;
    
    if (empty($sortOrder))
      return;
    
    if (array_key_exists($sortOrder, $ebay_map_sort_browseapi))
      $this->sortOrder = $sortOrder;  
  }
  
  
  public function setCategories(string $category)
  {
    $this->categoryIds = array();
    if (empty($category))
      $category = get_option('fubaby_ebayDefCategory');

    $categoryIdStrs = explode(',', $category);
    foreach ($categoryIdStrs as $cat)
    {
      $i = intval($cat);
      if ( $i != 0 )
        $this->categoryIds[] = $i;
      
      if (count($this->categoryIds) >= fuEbayBrowseSearchApiCall::EBAY_CATEGORIES_MAX)
        break;
    }
  }
  
  public function setAspect($name, $value)
  {
    $this->aspectName = urlencode($name);
    $this->aspectValue = urlencode($value);
  }

  protected function constructRequest()
  {
    // round limit up to nearest 12. More likely to hit cache on future variations
    $this->limit = ceil($this->limit / 12) * 12;

    $this->requestUri = $this->getEndpoint() . "search?";
    $this->requestUri .= "q=" . urlencode($this->query);
    $this->requestUri .= "&limit={$this->limit}";
    $this->requestUri .= "&sort={$this->sortOrder}";
    if (is_countable($this->categoryIds) && count($this->categoryIds) > 0)
      $this->requestUri .= "&categoryIds=" . urlencode(implode(',', $this->categoryIds));
    if (is_countable($this->sellers) && count($this->sellers) > 0)
      $this->requestUri .= "&sellers=" . urlencode(implode("|", $this->sellers));
    if ($this->minPrice > 0)
      $this->requestUri .= "&minPrice={$this->minPrice}";
      if ($this->maxPrice > 0)
      $this->requestUri .= "&maxPrice={$this->maxPrice}";

/*
    if (!empty($this->aspectName) && !empty($this->aspectValue))
    {
      $this->requestUri .= "&aspectFilter.aspectName={$this->aspectName}";
      $this->requestUri .= "&aspectFilter.aspectValueName={$this->aspectValue}";
    }
       */

    parent::constructRequest();

  }

  protected function createListingGrid($resp)
  {
    global $ebay_globalid_siteurls;
    if ($resp != null)
    {
      $titleUrl = isset($resp->itemSummaries) ? $resp->itemSummaries[0]->itemAffiliateWebUrl : $ebay_globalid_siteurls[$this->globalId];
  	  $this->genTableStart($titleUrl);
  
  	  if (isset($resp->itemSummaries))
  	  {
        $this->resultCount = count($resp->itemSummaries);
  		  foreach($resp->itemSummaries as $item)
  		  {
          $ebayItem = $this->initEbayListing($item);
          $this->genTableItem($ebayItem->GetListingAsHtml($this->picWidth, fuEbayShow::SearchListing | fuEbayShow::Store));
  		  }
  	  }
  	  
  	  if ($this->resultCount == 0)
  		  $this->result .= get_option('fubaby_ebayEmptySearchMsg');
  	  
   	  $this->genTableEnd();
    }      
  }

  // Init eBay Listing class from a search result item from API call.
  protected function initEbayListing($item)
  {
    global $ebay_globalid_currencies;
  
    $ebayItem = $this->createEbayListing();
    $ebayItem->Currency = $ebay_globalid_currencies[$this->globalId];
    $ebayItem->Arrangement = $this->arrangement;
    
    $ebayItem->ItemId = (string)$item->itemId;
    $ebayItem->Title = (string)$item->title;
    $ebayItem->Url = (string)$item->itemAffiliateWebUrl;
    $ebayItem->Sold = false;
    $ebayItem->PicHuge = (string)$item->thumbnailImages[0]->imageUrl;      // 1000px
    $ebayItem->PicLarge = (string)$item->thumbnailImages[0]->imageUrl;     // 1000px
    $ebayItem->GalleryLarge = (string)$item->image->imageUrl;     // 225px
    $ebayItem->GalleryMedium = (string)$item->image->imageUrl;     // 225px
    $ebayItem->GallerySmall = (string)$item->image->imageUrl;     // 225px

    $ebayItem->AuctionBids = isset($item->bidCount) ? (int)$item->bidCount : null;
    $ebayItem->EndTime = isset($item->itemEndDate) ? (string)$item->itemEndDate : null;
    $ebayItem->Active = isset($item->itemEndDate) ? strtotime($ebayItem->EndTime) > time() : true;
    $ebayItem->Location = isset($item->itemLocation->city) ? "{$item->itemLocation->city}, {$item->itemLocation->country}" : $item->itemLocation->country;
    $ebayItem->Feedback = (int)$item->seller->feedbackScore;
    $ebayItem->FeedbackPercent = isset($item->seller->feedbackPercentage) ? intval($item->seller->feedbackPercentage) : 100;
  
    if (in_array("AUCTION", $item->buyingOptions))
    {
      $ebayItem->Price = $item->currentBidPrice->value;
      $ebayItem->PriceConv = $item->currentBidPrice->value;

      if (in_array("FIXED_PRICE", $item->buyingOptions))
      {
        $ebayItem->PriceBIN = $item->price->value;
        $ebayItem->PriceBINConv = $item->price->value;
        $ebayItem->ListingType = "AuctionWithBIN"; // Auction + Fixed price = AuctionWithBIN
      }
      else
        $ebayItem->ListingType = "Auction";
    }
    
    // Auctions with BIN/Best Offers
    if (in_array("FIXED_PRICE", $item->buyingOptions) && !in_array("AUCTION", $item->buyingOptions))
    {
      $ebayItem->Price = $ebayItem->PriceBIN = $item->price->value;
      $ebayItem->PriceConv = $ebayItem->PriceBINConv = $item->price->value;
      $ebayItem->ListingType = "FixedPrice";
    }
    
    if (in_array("BEST_OFFER", $item->buyingOptions))
      $ebayItem->BINBestOffer = true;
     
    if ($item->seller->sellerAccountType == "BUSINESS" && isset($item->seller->username))
      $ebayItem->StoreName = (string)$item->seller->username;
    
    return $ebayItem;
  }
        

  protected function createDeferredLoadingStub()
  {
    global $ebay_globalid_siteurls;
    global $fuEbayTableSlideId;
    
    $stubId = "fu_ebay_" . parent::$id++;

    $this->genTableStart($ebay_globalid_siteurls[$this->globalId]);
    $this->result .= "Loading...";
    $this->genTableEnd();

    // Recreate result with above embedded within it. Thus no concatonate onto result.
    $this->result = '<div id="'.$stubId.'">'.$this->result.'</div>
    <script>
    var $content_'.$stubId.' = jQuery("#'.$stubId.'");
    jQuery(document).load(jQuery.ajax({
      type : "post",
      url : "'.admin_url('admin-ajax.php').'",
      data : {
          action : "fu_ebay_load_search",
          fu_ebay_title : '.json_encode($this->title).',
          fu_ebay_query : '.json_encode($this->query).',
          fu_ebay_limit : "'.$this->limit.'",
          fu_ebay_sort : "'.$this->sortOrder.'",
          fu_ebay_categoryIds : '.json_encode(implode(',', $this->categoryIds)).',
          fu_ebay_sellers : '.json_encode(implode(',', $this->sellers)).',
          fu_ebay_minPrice : "'.$this->minPrice.'",
          fu_ebay_maxPrice : "'.$this->maxPrice.'",
          fu_ebay_picwidth : "'.$this->picWidth.'",
          fu_ebay_cols : "'.$this->columns.'",
          fu_ebay_rows : "'.$this->rows.'",
          fu_ebay_slideshow : "'.$this->slideShow.'",
          fu_ebay_slides : "'.$this->slides.'",
          fu_ebay_slideId : "'.$fuEbayTableSlideId.'",
          fu_ebay_customId : "'.$this->customId.'",
          nonce : "'.wp_create_nonce("shortcode-search").'",
      },
      success : function( response ) {
          $content_'.$stubId.'.html( response );
      },
      error: function(e) {
        console.log("Fast eBay Listings deferred loading error: " + e);
      }
    }));
    </script>
    ';
  }

  protected function validateArgs()
  {
    parent::validateArgs();
    self::validate();

    if ($this->picWidth <= 0) $this->picWidth = get_option('fubaby_ebayPicWidthList');	

    if (empty($this->query) && is_countable($this->categoryIds) && count($this->categoryIds) == 0)
    {
      $this->errorLog(__(": query or categories must be set, both must not be blank."));
      return false;
    }

    if (empty($this->sortOrder)) $this->sortOrder = get_option('fubaby_ebayDefSort');
    
    // Limits set by Browse Item_Summary/Search API call
    if ($this->limit > fuEbayBrowseSearchApiCall::EBAY_LIMIT_MAX) $this->limit = fuEbayBrowseSearchApiCall::EBAY_LIMIT_MAX;
    
    if (is_countable($this->categoryIds) && count($this->categoryIds) > fuEbayBrowseSearchApiCall::EBAY_CATEGORIES_MAX)
    {
      $this->errorLog(__(": too many category ids, max = " . fuEbayBrowseSearchApiCall::EBAY_CATEGORIES_MAX));
      return false;
    }
 
    $this->minPrice = floatval($this->minPrice);
    $this->maxPrice = floatval($this->maxPrice);

    return true;
  }    
}

// Main AJAX entry function
function fu_ebay_load_search() 
{
  global $fuEbayTableSlideId;
  try
  {
    if (!check_ajax_referer('shortcode-search', 'nonce', false)) 
    {
      $this->errorLog('Nonce check failed');
      die;
    }

    $apicall = new fuEbayBrowseSearchApiCall(stripslashes($_POST['fu_ebay_title']));
    if (isset($_POST['fu_ebay_query'])) 
      $apicall->setQuery(stripslashes($_POST['fu_ebay_query']));
    if (isset($_POST['fu_ebay_sort'])) 
      $apicall->setSortOrder($_POST['fu_ebay_sort']);
    if (isset($_POST['fu_ebay_categoryIds'])) 
      $apicall->setCategories(stripslashes($_POST['fu_ebay_categoryIds']));
    if (isset($_POST['fu_ebay_sellers'])) 
      $apicall->setSellers(stripslashes($_POST['fu_ebay_sellers']));
    if (isset($_POST['fu_ebay_minPrice'])) 
      $apicall->minPrice = $_POST['fu_ebay_minPrice'];
    if (isset($_POST['fu_ebay_maxPrice'])) 
      $apicall->maxPrice = $_POST['fu_ebay_maxPrice'];
    if (isset($_POST['fu_ebay_picwidth'])) 
      $apicall->picWidth = $_POST['fu_ebay_picwidth'];
    if (isset($_POST['fu_ebay_cols']) && isset($_POST['fu_ebay_rows'])) 
      $apicall->setTableSize($_POST['fu_ebay_cols'], $_POST['fu_ebay_rows']);
    if (isset($_POST['fu_ebay_slideshow']) && isset($_POST['fu_ebay_slides'])) 
      $apicall->setSlideShow($_POST['fu_ebay_slideshow'], $_POST['fu_ebay_slides']); 
    if (isset($_POST['fu_ebay_customId'])) $apicall->customId = $_POST['fu_ebay_customId'];

    // override slide ids to avoid clashing with other slides on pages.
    $fuEbayTableSlideId = intval($_POST['fu_ebay_slideId']);

    echo $apicall->call(true);

  } catch(TypeError $e)
  {
    echo $e->getMessage();
  }
  die();
}

add_action( 'wp_ajax_fu_ebay_load_search', 'fu_ebay_load_search' );
add_action( 'wp_ajax_nopriv_fu_ebay_load_search', 'fu_ebay_load_search' );