( function( wp ) {
	/**
	 * Registers a new block provided a unique name and an object defining its behavior.
	 * @see https://github.com/WordPress/gutenberg/tree/master/blocks#api
	 */
	var registerBlockType = wp.blocks.registerBlockType;
	/**
	 * Returns a new element of given type. Element is an abstraction layer atop React.
	 * @see https://github.com/WordPress/gutenberg/tree/master/element#element
	 */
	var el = wp.element.createElement;
	/**
	 * Retrieves the translation of text.
	 * @see https://github.com/WordPress/gutenberg/tree/master/i18n#api
	 */
	var __ = wp.i18n.__;

	var blocks = wp.blocks;
	
	/**
	 * Import stuff for the Inspector Controls 
	 */
	var InspectorControls = wp.editor.InspectorControls;
	var PlainText = wp.editor.PlainText;
	var PanelBody = wp.components.PanelBody;
	var SelectControl = wp.components.SelectControl;
	var TextControl = wp.components.TextControl;
	
	/**
	 * Every block starts by registering a new block type definition.
	 * @see https://wordpress.org/gutenberg/handbook/block-api/
	 */
	registerBlockType( 'fast-ebay-listings/item', {
		/**
		 * This is the display title for your block, which can be translated with `i18n` functions.
		 * The block inserter will show this name.
		 */
		title: __( 'Fast eBay Item' ),

		/**
		 * An icon property should be specified to make it easier to identify a block.
		 * These can be any of WordPress’ Dashicons, or a custom svg element.
		 */
		icon: 'store',

		/**
		 * Blocks are grouped into categories to help users browse and discover them.
		 * The categories provided by core are `common`, `embed`, `formatting`, `layout` and `widgets`.
		 */
		category: 'common',

		/**
		 * Optional block extended support features.
		 */
		supports: {
			// Removes support for an HTML mode.
			html: false,
		},
		
		/**
		 * Block attributes 
		 */
		attributes: { // Necessary for saving block content.
			item: { type: 'string' },
			picwidth: { type: 'string' },
			customid: { type: 'string' }
		},

		/**
		 * The edit function describes the structure of your block in the context of the editor.
		 * This represents what the editor will render when the block is used.
		 * @see https://wordpress.org/gutenberg/handbook/block-edit-save/#edit
		 *
		 * @param {Object} [props] Properties passed from the editor.
		 * @return {Element}       Element to render.
		 */
		edit: function( props ) {
			var attributes = props.attributes;
			var item = attributes.item != null && attributes.item.length > 0 ? attributes.item : __( '' );
			var customid = attributes.customid != null && attributes.customid.length > 0 ? attributes.customid : '';
			
			/* Event handlers */

			/* Return the edit content */
			return [
					!! props.isSelected && el(InspectorControls, { key: 'inspector' },
						el( 'div', { className: 'components-block-description' }),
						
						el(PanelBody, 
								{ 
									title: __( 'eBay Query Options' ),
									initialOpen: true
								},
								el(TextControl,
									{
										type: 'text',
										label: __( 'Item Id' ),
										value: item,
										onChange: function( value ) {
										    props.setAttributes( { item: value } );
										},
									}
								),
								el(TextControl,
									{
										type: 'text',
										label: __( 'Custom Id' ),
										placeholder: __( '' ),
										value: customid,
										onChange: function ( value ) {
											props.setAttributes( { customid: value } );
										},
									}
								),
							),												
						el(PanelBody, 
							{ 
								title: __( 'Presentation Options' ),
								initialOpen: true 
							},
							el(TextControl,
								{
									type: 'number',
									label: __( 'Picture Width' ),
									value: props.attributes.picwidth,
									onChange: function ( value ) {
										props.setAttributes( { picwidth: value } );
									},
								}
							)
						)
					),		
					el( 'div', 
						{ className: 'fu_ebay_results_container' }
						),
						el( 'img',
							{ src: fuScript.pluginsUrl + '/blocks/images/ebayitempanel.jpg', width: '701', height: '126' } 
						),						
					];
		},

		/**
		 * The save function defines the way in which the different attributes should be combined
		 * into the final markup, which is then serialized by Gutenberg into `post_content`.
		 * @see https://wordpress.org/gutenberg/handbook/block-edit-save/#save
		 *
		 * @return {Element}       Element to render.
		 */
		save: function( props ) {
			// Rendering in PHP.
			return null;
		}
	} );
} )(
	window.wp
);
