( function( wp ) {
	/**
	 * Registers a new block provided a unique name and an object defining its behavior.
	 * @see https://github.com/WordPress/gutenberg/tree/master/blocks#api
	 */
	var registerBlockType = wp.blocks.registerBlockType;
	/**
	 * Returns a new element of given type. Element is an abstraction layer atop React.
	 * @see https://github.com/WordPress/gutenberg/tree/master/element#element
	 */
	var el = wp.element.createElement;
	/**
	 * Retrieves the translation of text.
	 * @see https://github.com/WordPress/gutenberg/tree/master/i18n#api
	 */
	var __ = wp.i18n.__;

	var blocks = wp.blocks;
	
	/**
	 * Import stuff for the Inspector Controls 
	 */
	var InspectorControls = wp.editor.InspectorControls;
	var PlainText = wp.editor.PlainText;
	var PanelBody = wp.components.PanelBody;
	var SelectControl = wp.components.SelectControl;
	var TextControl = wp.components.TextControl;
	
	/**
	 * Every block starts by registering a new block type definition.
	 * @see https://wordpress.org/gutenberg/handbook/block-api/
	 */
	registerBlockType( 'fast-ebay-listings/search', {
		/**
		 * This is the display title for your block, which can be translated with `i18n` functions.
		 * The block inserter will show this name.
		 */
		title: __( 'Fast eBay Search' ),

		/**
		 * An icon property should be specified to make it easier to identify a block.
		 * These can be any of WordPress’ Dashicons, or a custom svg element.
		 */
		icon: 'store',

		/**
		 * Blocks are grouped into categories to help users browse and discover them.
		 * The categories provided by core are `common`, `embed`, `formatting`, `layout` and `widgets`.
		 */
		category: 'common',

		/**
		 * Optional block extended support features.
		 */
		supports: {
			// Removes support for an HTML mode.
			html: false,
		},
		
		/**
		 * Block attributes 
		 */
		attributes: { // Necessary for saving block content.
			title: { type: 'string' },
			query: { type: 'string' },
			category: { type: 'string' },
			seller: { type: 'string' },
			sort: { type: 'string' },
			picwidth: { type: 'string' },
			minprice: { ype: 'string' },
			maxprice: { type: 'string' },
			columns: { type: 'string' },
			rows: { type: 'string' },
			slideshow: { type: 'string' },
			slides: { type: 'string' },
			customid: { type: 'string' }
		},

		/**
		 * The edit function describes the structure of your block in the context of the editor.
		 * This represents what the editor will render when the block is used.
		 * @see https://wordpress.org/gutenberg/handbook/block-edit-save/#edit
		 *
		 * @param {Object} [props] Properties passed from the editor.
		 * @return {Element}       Element to render.
		 */
		edit: function( props ) {
			var attributes = props.attributes;
			var title = attributes.title != null && attributes.title.length > 0 ? attributes.title : __( 'Fast eBay Search' );
			var query = attributes.query != null && attributes.query.length > 0 ? attributes.query : '';
			var customid = attributes.customid != null && attributes.customid.length > 0 ? attributes.customid : '';
			
			/* Event handlers */

			/* Return the edit content */
			return [
					!! props.isSelected && el(InspectorControls, { key: 'inspector' },
						el( 'div', { className: 'components-block-description' }),
						
						el(PanelBody, 
							{ 
								title: __( 'eBay Query Options' ),
								initialOpen: false
							},
							el(TextControl,
									{
										type: 'text',
										label: __( 'Query' ),
										placeholder: __( 'nice things' ),
										value: query,
										onChange: function ( value ) {
											props.setAttributes( { query: value } );
										},
									}
								),
							el(TextControl,
								{
									type: 'text',
									label: __( 'Category' ),
									value: props.attributes.category,
									onChange: function ( value ) {
										props.setAttributes( { category: value } );
									},
								}
							),
							el(TextControl,
								{
									type: 'text',
									label: __( 'Seller' ),
									value: props.attributes.seller,
									onChange: function ( value ) {
										props.setAttributes( { seller: value } );
									},
								}
							),
							el(SelectControl,
								{
									type: 'text',
									label: __( 'Sorting' ),
									value: props.attributes.sort,
									onChange: function ( value ) {
										props.setAttributes( { sort: value } );
									},
									options: [						           
										{label: 'Best Match', value: 'BestMatch'},
										{label: 'Fewest Bids First', value: 'BidCountFewest'},
										{label: 'Most Bids First', value: 'BidCountMost'},
										{label: 'Highest Price First', value: 'CurrentPriceHighest'},
										{label: 'Nearest First', value: 'DistanceNearest'},
										{label: 'Ending Soonest', value: 'EndTimeSoonest'},
										{label: 'Highest Price + P&P First', value: 'PricePlusShippingHighest'},
										{label: 'Lowest Price + P&P First', value: 'PricePlusShippingLowest'},
										{label: 'Newest First', value: 'StartTimeNewest'}
									],
								}
							),
							el(TextControl,
								{
									type: 'number',
									label: __( 'Min Price' ),
									value: props.attributes.minprice,
									onChange: function ( value ) {
										props.setAttributes( { minprice: value } );
									},
								}
							),							
							el(TextControl,
								{
									type: 'number',
									label: __( 'Max Price' ),
									value: props.attributes.maxprice,
									onChange: function ( value ) {
										props.setAttributes( { maxprice: value } );
									},
								}
							),
							el(TextControl,
								{
									type: 'text',
									label: __( 'Custom Id' ),
									placeholder: __( '' ),
									value: customid,
									onChange: function ( value ) {
										props.setAttributes( { customid: value } );
									},
								}
							),						
						),
					el(PanelBody, 
						{ 
							title: __( 'Presentation Options' ),
							initialOpen: false 
						},
						el(TextControl,
							{
								type: 'number',
								label: __( 'Picture Width' ),
								value: props.attributes.picwidth,
								onChange: function ( value ) {
									props.setAttributes( { picwidth: value } );
								},
							}
						),
						el(TextControl,
							{
								type: 'number',
								label: __( 'Columns' ),
								value: props.attributes.columns,
								onChange: function ( value ) {
									props.setAttributes( { columns: value } );
								},
							}
						),									
						el(TextControl,
							{
								type: 'number',
								label: __( 'Rows' ),
								value: props.attributes.rows,
								onChange: function ( value ) {
									props.setAttributes( { rows: value } );
								},
							}
						),
						el(SelectControl,
							{
								type: 'text',
								label: __( 'Slideshow Style' ),
								value: props.attributes.slideshow,
								onChange: function ( value ) {
									props.setAttributes( { slideshow: value } );
								},
								options: [						           
										{label: 'Settings Default', value: ''},
										{label: 'None', value: 'None'},
										{label: 'Manual', value: 'Manual'},
										{label: 'Auto', value: 'Auto'},
									],
							}
						),			
						el(TextControl,
							{
								type: 'number',
								label: __( 'Number of slides' ),
								value: props.attributes.slides,
								onChange: function ( value ) {
									props.setAttributes( { slides: value } );
								},
							}
						)	
					)
					),		
					el( 'div', 
						{ className: 'fu_ebay_results_container' },
						el( 'div', 
							{ className: 'fu_ebay_results_title' },						
							el( 'a',
								{ href: '#' },
								el( 'img',
									{ src: fuScript.pluginsUrl + '/US/Right Now 108x45.gif', width: '108', height: '45' } 
									)),
							el( 'a',
								{ href: '#' },										
								el( PlainText, 
									{
										tagName: 'p',
										inline: true,
										className: 'fu_ebay_results_title_editor',
										placeholder: __( 'Fast eBay Search' ),
										value: title,
										onChange: function( value ) {
										    props.setAttributes( { title: value } );
										  }										  
									}
								)
							)			
						)
					),
					el( 'img',
						{ src: fuScript.pluginsUrl + '/blocks/images/ebaysearchpanel.jpg', width: '644', height: '78' } 
					),						
				];
		},

		/**
		 * The save function defines the way in which the different attributes should be combined
		 * into the final markup, which is then serialized by Gutenberg into `post_content`.
		 * @see https://wordpress.org/gutenberg/handbook/block-edit-save/#save
		 *
		 * @return {Element}       Element to render.
		 */
		save: function( props ) {
			// Rendering in PHP.
			return null;
		}
	} );
} )(
	window.wp
);
